/**
far away - a free space shooter game
Copyright (C) 2005  Alexander Clausen, Dirk Rathlev, Stefan Toller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

// constants

// size of images[]
#define	MAX_IMAGES	20
// number of loaded tile images (only for testing, should be a variable)
#define NUM_IMAGES	14

// default screen resolution
#define SCREEN_W	800
#define SCREEN_H	600

// animation types
#define ANIM_NORMAL	0
#define ANIM_MOVING	0
#define ANIM_ATTACKING	1
#define ANIM_DIEING	2

#define MAX_OBJECT_TYPES	20
#define OBJ_TYPE_NULL		0
#define OBJ_TYPE_PLAYER		1
#define OBJ_TYPE_ENEMY		2
#define OBJ_TYPE_SHOT		3
#define OBJ_TYPE_STONE		5
#define OBJ_TYPE_PWRUP_MED	6
#define OBJ_TYPE_PWRUP_L1	7
#define OBJ_TYPE_PWRUP_L2	8
#define OBJ_TYPE_PWRUP_L3	9
#define OBJ_TYPE_PWRUP_REGENERATE	10
#define OBJ_TYPE_PWRUP_SPEED	11

#define	NUM_SOUNDS		5

#define	GS_NULL			0
#define GS_START_MENU		1
#define	GS_GAME			2
#define GS_MENU			3
#define	GS_GAMEOVER		4
#define	GS_QUIT			5
#define	GS_SPLASH		6
#define	GS_NEXT_LEVEL		7

// different types of object states
#define	OBJ_STATE_MOVING	1
#define	OBJ_STATE_DYING	2

// number of images for the UI
#define	MAX_MENU_ICONS		2

// number of menu items in start menu screen
#define	NUM_START_MENU_ITEMS	2
// number of menu items in ingame menu screen
#define	NUM_INGAME_MENU_ITEMS	2
// number of other UI graphics
#define	MAX_UI_GRAPHICS		4
// number of splash screens
#define	MAX_SPLASH_SCREENS	3

#define	MENU_START		0
#define	MENU_RETURN		1
#define	MENU_QUIT		2

#define WEAPON_NULL		0
#define	WEAPON_NORMAL		1
#define WEAPON_DISPERSION	2

#define MOVEMENT_NONE		0
#define MOVEMENT_LINEAR		1
#define MOVEMENT_DISPERSE	2
#define MOVEMENT_DAMPED		3

// how many weapons maximal for each object
#define	MAX_ATTACHED_WEAPONS		5
// how many weapon types? size of WeaponTypes array in objects.h
#define	MAX_WEAPON_TYPES		20

// how many maps maximal for each campaign
#define	MAX_MAPS_PER_CAMPAIGN		10

#define	TARGET_TIME		1
#define	TARGET_WAY		2
#define TARGET_BOSS		3

struct SMouse
{    
	// state of left and right mouse button:
	// true is pressed, false is released
	bool	buttonl;
	bool	buttonr;
	
	// mouse position
	int	x,y;
};

struct SMenuItem
{
	// indictaes whether menu item is visible
	bool	bvisible;
	// type of the action (new game, quit, ...)
	int	nType;
	// position on the screen
	int	x,y;
	// width and height
	int	w,h;
	// image number in menu_icons[]
	int	nImage;
};

struct SWeaponType
{
	// indicates the fire rate of the weapon. fwcounter (SWeapon) must be > than fwtime
	float	fwtime;
	// type of weapon, definitions in globals.h (WEAPON_*)
	int	nWType;
	// damage
	int	damage;
	// index number of the object which represent the projectile
	int	nProjObj;
};

struct SWeapon
{
	// internal counter for weapon firing
	// must be > than fwtime (SWeaponType) to fire the gun
	float	fwcounter;
	// position of weapon in relation to the top left corner of an object
	int	x,y;
	// pointer to weapon type
	SWeaponType	*WeaponType;
};

struct SConfig
{
	bool	bDoubleBuffering;
	bool	bHWSurface;
	bool	bFullscreen;
	bool	bSoundEnabled;
	bool	bMusicEnabled;
	int	nSampleRate;
	int	nSoundBuffer;
};

