/**
far away - a free space shooter game
Copyright (C) 2005  Alexander Clausen, Dirk Rathlev, Stefan Toller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "SDL.h"	
#include "SDL_image.h"	
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "faraway.h"

extern faMain* g_pfaMain;

int Objects::objects_init()
{
	if (load_objecttypes("data/objtypes"))
	{
		printf("loading objecttypes failed\n");
	}
	
	if (load_weapontypes("data/weapons"))
	{
		printf("loading weapons failed\n");
	}
	
	srand(time(NULL));

	return 0;
}

int Objects::add_objecttype(SObjectType *ObjType)
{
	int i;
	
	for (i=1; i<MAX_OBJECT_TYPES; i++)
	{
		if (lObjectTypes[i].nType == 0)
		{
			lObjectTypes[i] = *ObjType;
			printf("added object type %d\n",i);
			return i;
		}
	}
	
	
	return -1;
}

int Objects::del_objecttype(int nObjType)
{
	lObjectTypes[nObjType].nType = 0;
	
	return 0;
}


int Objects::add_object(SObject *Obj)
{
	unsigned int i, size = lObjects.size();
	
		for (i=0; i<size; i++)
	{
		if (!lObjects[i].ObjType)
		{
			lObjects[i] = *Obj;
			//printf("added object at %d, position: %f/%f\n",i,Obj->x,Obj->y);
			return i;
		}
	}
	
	srand(time(NULL));
	
	lObjects.push_back(*Obj);
	
	//printf("added object at %d, position: %f/%f\n",i,Obj->x,Obj->y);
	
	return i;
}

int Objects::print_debug(void)
{
	unsigned int i, size = lObjects.size();
	SObject	obj;
	// list empty?
	for (i=0; i<size; i++)
	{
			obj = lObjects[i];
			printf("x: %f y: %f\n",obj.x,obj.y);
	}
	return 0;
}

int Objects::del_object(int nObj)
{
	unsigned int i;
	
	for (i=0; i<lObjects[nObj].numWeapons; i++)
	{
		if (lObjects[nObj].Weapons[i] != NULL)
		{
			free(lObjects[nObj].Weapons[i]);
			
		}
	}
	
	lObjects[nObj].ObjType = NULL;
		
	//printf("deleted object at %d, position: %f/%f\n",nObj,lObjects[nObj].x,lObjects[nObj].y);
	return 0;
}

int Objects::exit_objects(void)
{
	unsigned int	i, size = lObjects.size();
	
	for (i=0; i<size; i++)
	{
		if (NULL != lObjects[i].ObjType)
		{
			del_object(i);
		}
	}
	
	lObjects.clear();
	
	return 0;
}

int Objects::ai_object(int nObj)
{
	// only process ai if object is on the screen
	if (lObjects[nObj].x > SCREEN_W && lObjects[nObj].ObjType->nType != OBJ_TYPE_SHOT)
	{
		return 0;
	}
	
	// don't process dying objects
	if (lObjects[nObj].nState == OBJ_STATE_DYING)
	{
		return 0;
	}
	
	switch(lObjects[nObj].ObjType->nType)
	{
		case OBJ_TYPE_ENEMY:
			if ((lObjects[nObj].x - lObjects[num_player1].x) < 700.0f)
			{
				fire_weapons(nObj);
			}
			lObjects[nObj].speed_x = -lObjects[nObj].ObjType->speed;
			break;
		case OBJ_TYPE_SHOT:
			if (lObjects[nObj].nState == OBJ_STATE_DYING)
			{
				destroy_object(nObj);
				break;
			}
			// destroy projectiles which have left the screen or arrived their lifetime
			if ( (lObjects[nObj].fcounter >= lObjects[nObj].ObjType->flifetime) || (lObjects[nObj].x > SCREEN_W) || (lObjects[nObj].x+lObjects[nObj].ObjType->w < 0) )
			{
				destroy_object(nObj);
				break;
			}
			break;
		case OBJ_TYPE_STONE:
			lObjects[nObj].speed_x = -lObjects[nObj].ObjType->speed;
			break;
		default:
			break;
	}
	return 0;
}

int Objects::move_object(unsigned int nObj)
{
	// don't move dieing objects
	if (lObjects[nObj].nState == OBJ_STATE_DYING)
	{
		return 0;
	}
	
	// when object not on the screen, don't process collisions but move it with game_speed
	if (lObjects[nObj].x > SCREEN_W)
	{
		// move all objects except the player toward the left
		if (nObj != num_player1)
		{
			lObjects[nObj].x -= (g_pfaMain->game_speed*g_pfaMain->fTime);
		}
		return 1;
	}
	
	struct SObject *obj = &lObjects[nObj];
	
	/* speed/position update */
	switch(obj->ObjType->movement) {
		case MOVEMENT_DAMPED:
			// fixme: hardcoded damping factor
			obj->speed_x -= (obj->speed_x/50);
			obj->speed_y -= (obj->speed_y/50);
			if(obj->speed_x > -obj->ObjType->speed) {
				obj->speed_x = -obj->ObjType->speed;
			}
			if (obj->x+obj->ObjType->w > SCREEN_W) {
				//obj->x = SCREEN_W-obj->ObjType->w;
				obj->speed_x = -(obj->speed_x + 10);
			}
			if (obj->y < 0) {
				//obj->y = 0;
				obj->speed_y = -(obj->speed_y + 10);
			} else if (obj->y+obj->ObjType->h > SCREEN_H) {
				//obj->y = SCREEN_H-obj->ObjType->h;
				obj->speed_y = -(obj->speed_y + 10);
			}
			/* fallthrough */
		case MOVEMENT_LINEAR:
		case MOVEMENT_DISPERSE:
			obj->x += obj->speed_x * g_pfaMain->fTime;
			obj->y += obj->speed_y * g_pfaMain->fTime;
			break;
		case MOVEMENT_NONE:
			/* don't move */
			break;
	}
	
	// delete all objects which have left the screen on the left
	if (lObjects[nObj].x < -lObjects[nObj].ObjType->w)
	{
		del_object(nObj);
		return 0;
	}
	
	unsigned int i, size = lObjects.size();

	for (i=0; i<size; i++)
	{
		if ( (lObjects[i].ObjType) && (nObj != i) && check_collision_rect(nObj,i) == 1)
		{
			struct SObject *o1, *o2;
			unsigned int o1type, o2type;

			o1 = &lObjects[nObj];
			o2 = &lObjects[i];

			o1type = o1->ObjType->nType;
			o2type = o2->ObjType->nType;
			
			switch(o1type) {
				case OBJ_TYPE_PLAYER:
					if(o2type == OBJ_TYPE_PWRUP_MED) {
						action_health(nObj, o2->hp);
						action_kill(i, false);
						return 1;
					}
					if(o2type == OBJ_TYPE_PWRUP_L1
						|| o2type == OBJ_TYPE_PWRUP_L2
						|| o2type == OBJ_TYPE_PWRUP_L3
						|| o2type == OBJ_TYPE_PWRUP_SPEED)
					{
						action_pwrup(nObj, i);
						action_kill(i, false);
						return 1;
					}
					break;
				case OBJ_TYPE_PWRUP_MED:
					if(o2type == OBJ_TYPE_PLAYER) {
						action_health(i, o1->hp);
						action_kill(nObj, false);
					}
					break;
				case OBJ_TYPE_PWRUP_L1:
				case OBJ_TYPE_PWRUP_L2:
				case OBJ_TYPE_PWRUP_L3:
				case OBJ_TYPE_PWRUP_SPEED:
				case OBJ_TYPE_PWRUP_REGENERATE:
					if(o2type == OBJ_TYPE_PLAYER) {
						action_pwrup(i, nObj);
						action_kill(nObj, false);
					}
					break;
				default:
					action_collide(nObj, i);
					break;
			}
			return 1;
		}
	}
	return 0;
}

int Objects::animate_object(int nObj)
{
	if (!lObjects[nObj].ObjType)
	{
		return 1;
	}
	lObjects[nObj].animtime += (lObjects[nObj].ObjType->animspeed*g_pfaMain->fTime);
	if (lObjects[nObj].animtime >= 1.0f)
	{
		lObjects[nObj].animtime = 0.0f;
		lObjects[nObj].nanim++;
		if (lObjects[nObj].nanim > lObjects[nObj].ObjType->maxanim)
		{
			if (lObjects[nObj].nState == OBJ_STATE_DYING)
			{
				destroy_object(nObj);
				return 0;
			}
			lObjects[nObj].nanim = 0;
		}
	}
	return 0;
}

int Objects::process_objects(void)
{
	unsigned int i,j,size;
	
	// set speed_x/speed_y to 0 (do this only for player)
	lObjects[num_player1].speed_x = 0;
	lObjects[num_player1].speed_y = 0;
	if (g_pfaMain->g_pInput->keys[SDLK_UP] == SDL_PRESSED)
	{
		lObjects[num_player1].speed_y = -lObjects[num_player1].ObjType->speed;
	}
	if (g_pfaMain->g_pInput->keys[SDLK_DOWN] == SDL_PRESSED)
	{
		lObjects[num_player1].speed_y = lObjects[num_player1].ObjType->speed;
	}
	if (g_pfaMain->g_pInput->keys[SDLK_LEFT] == SDL_PRESSED)
	{
		lObjects[num_player1].speed_x = -lObjects[num_player1].ObjType->speed;
	}
	if (g_pfaMain->g_pInput->keys[SDLK_RIGHT] == SDL_PRESSED)
	{
		lObjects[num_player1].speed_x = lObjects[num_player1].ObjType->speed;
	}

	if(lObjects[num_player1].pwrup_speed_seconds_left > 0) {
		lObjects[num_player1].speed_x *= 1.3;
		lObjects[num_player1].speed_y *= 1.3;
		lObjects[num_player1].pwrup_speed_seconds_left -= g_pfaMain->fTime;
	}

	if(lObjects[num_player1].pwrup_regenerate_seconds_left > 0) {
		lObjects[num_player1].hp += lObjects[num_player1].ObjType->maxhp / 100;
		if(lObjects[num_player1].hp > lObjects[num_player1].ObjType->maxhp)
		{
			lObjects[num_player1].hp = lObjects[num_player1].ObjType->maxhp;
		}
		lObjects[num_player1].pwrup_regenerate_seconds_left -= g_pfaMain->fTime;
	}
	
	if (lObjects[num_player1].x < 0)
	{
		lObjects[num_player1].x = 0;
	}
	if (lObjects[num_player1].x+lObjects[num_player1].ObjType->w > SCREEN_W)
	{
		lObjects[num_player1].x = SCREEN_W-lObjects[num_player1].ObjType->w;
	}
	if (lObjects[num_player1].y < 0)
	{
		lObjects[num_player1].y = 0;
	}
	if (lObjects[num_player1].y+lObjects[num_player1].ObjType->h > SCREEN_H)
	{
		lObjects[num_player1].y = SCREEN_H-lObjects[num_player1].ObjType->h;
	}
	
	if ( (g_pfaMain->g_pInput->keys[SDLK_LCTRL] == SDL_PRESSED) || (g_pfaMain->g_pInput->keys[SDLK_RCTRL] == SDL_PRESSED) )
	{
		fire_weapons(num_player1);
	}

	size = lObjects.size();
	for (i=0; i<size; i++)
	{
		if (!lObjects[i].ObjType)
		{
			continue;
		}
		
		lObjects[i].fcounter += g_pfaMain->fTime;
		// no overflow!
		if (lObjects[i].fcounter > 300000000.0f)
		{
			lObjects[i].fcounter = 0.0f;
		}
		
		// increase counters of all existing weapons
		for (j=0; j<lObjects[i].numWeapons; j++)
		{
			lObjects[i].Weapons[j]->fwcounter += g_pfaMain->fTime;
			if (lObjects[i].Weapons[j]->fwcounter > 300000000.0f)
			{
				lObjects[i].Weapons[j]->fwcounter = 0.0f;
			}
		}

		ai_object(i);
		if (!lObjects[i].ObjType)
		{
			continue;
		}
		move_object(i);
		if (!lObjects[i].ObjType)
		{
			continue;
		}
		animate_object(i);
	}
	
	return 0;
}


	
unsigned int Objects::check_collision_rect(int nObj1, int nObj2)
{
	//printf("%d\n",lObjects[num_player1].nparent);
	// no collisions with dieing objects
	if ( (lObjects[nObj1].nState == OBJ_STATE_DYING) || (lObjects[nObj2].nState == OBJ_STATE_DYING) )
	{
		return 0;
	}
	// no collisions between lasers and their "parents"
	if ( (lObjects[nObj1].nparent == nObj2) || (lObjects[nObj2].nparent == nObj1) )
	{
		return 0;
	}
	
		
	// FixMe: bad hack
	// no collisions between powerups and enemies
	if ( (lObjects[nObj1].ObjType->nType > 5) && ((lObjects[nObj2].ObjType->nType == OBJ_TYPE_ENEMY) || (lObjects[nObj2].ObjType->nType == OBJ_TYPE_STONE)) ) {
		return 0;
	}
	if ( (lObjects[nObj2].ObjType->nType > 5) && ((lObjects[nObj1].ObjType->nType == OBJ_TYPE_ENEMY) || (lObjects[nObj1].ObjType->nType == OBJ_TYPE_STONE)) ) {
		return 0;
	}
	
	

	if ( (lObjects[nObj1].x+lObjects[nObj1].ObjType->w > lObjects[nObj2].x) && (lObjects[nObj1].x < lObjects[nObj2].x+lObjects[nObj2].ObjType->w) &&
		     (lObjects[nObj1].y+lObjects[nObj1].ObjType->h > lObjects[nObj2].y) && (lObjects[nObj1].y < lObjects[nObj2].y+lObjects[nObj2].ObjType->h) )
	{
		if ( (lObjects[nObj1].ObjType->nType == OBJ_TYPE_SHOT) && (lObjects[nObj2].ObjType->nType == OBJ_TYPE_SHOT) )
		{
			// no collisions between lasers
			return 0;
		}
		return 1;
	}
	return 0;
}
	
int Objects::create_object(SObject *Obj)
{
	int i;
	
	Obj->nanim = 0;
	Obj->fcounter = 0.0f;
	Obj->animtime = 0.0f;
	Obj->nanimtype = 0;

	Obj->nState = OBJ_STATE_MOVING;
	Obj->nparent = -1;
	
	Obj->pwrup_speed_seconds_left = 0;
	Obj->pwrup_regenerate_seconds_left = 0;
	
	
	for (i=Obj->numWeapons; i<MAX_ATTACHED_WEAPONS; i++)
	{
		Obj->Weapons[i] = NULL;
	}

	/* set initial speed */
	switch(Obj->ObjType->movement) {
		case MOVEMENT_NONE:
			Obj->speed_y = 0.0f;
			Obj->speed_x = 0.0f;
			break;
		case MOVEMENT_LINEAR:
			Obj->speed_y = 0.0f;
			Obj->speed_x = Obj->ObjType->speed;
			break;
		case MOVEMENT_DISPERSE:
			Obj->speed_y = (80.0f/get_random(-10,10));
			Obj->speed_x = Obj->ObjType->speed;
			break;
		case MOVEMENT_DAMPED:
			Obj->speed_y = (1000.0f/get_random(-10,10));
			Obj->speed_x = (1000.0f/get_random(-10,10));
			break;
		default:
			printf("unhandled movement type %d\n", Obj->ObjType->movement);
	}

	//printf("created: %f, %f, %f, %f, %d\n",Obj->x,Obj->y,Obj->speed_x,Obj->speed_y,Obj->ObjType->movement);
	return add_object(Obj);
}

int Objects::destroy_object(int nObj)
{
	//printf("destroy object %d\n",nObj);
	if (lObjects[nObj].ObjType->nType == OBJ_TYPE_PLAYER)
	{
		// show game over screen
		g_pfaMain->GameState = GS_SPLASH;
		g_pfaMain->continue_at = GS_GAMEOVER;
		g_pfaMain->splash = 0;
	}

	del_object(nObj);
	
	return 0;
}
	

int Objects::load_level(char* filename)
{
	unsigned int nObjects;
	FILE* file;
	int nObjType,x,y,hp,nWeapons,nWType;
	SObject	Obj;
	unsigned int i;
	int j;
	
	file = fopen(filename,"r");
	if (file == NULL)
	{
		return 1;
	}
	fscanf(file,"%d,%d,%d\n",&nObjects,&g_pfaMain->nTargetType,&g_pfaMain->level_length);
	printf("nObjects: %d, level_length: %d\n",nObjects,g_pfaMain->level_length);
	
	for (i=0; i<nObjects; i++)
	{
		fscanf(file,"%d,%d,%d,%d,%d\n",&nObjType,&x,&y,&hp,&nWeapons);
		printf("%d,%d,%d,%d,%d\n",nObjType,x,y,hp,nWeapons);

		Obj.ObjType = &lObjectTypes[nObjType];
		Obj.x = x;
		Obj.y = y;
		Obj.speed_x = 0.0f;
		Obj.speed_y = 0.0f;
		Obj.nanim = 0;
		if (hp == 0)
		{
			Obj.hp = Obj.ObjType->maxhp;
		}
		else
		{
			Obj.hp = hp;
		}

		Obj.numWeapons = nWeapons;
		
		for (j=0; j<nWeapons; j++)
		{		
			fscanf(file," %d,%d,%d\n",&nWType,&x,&y);
			printf("weapon: %d,%d,%d\n",nWType,x,y);		
			Obj.Weapons[j] = (SWeapon*)malloc(sizeof(struct SWeapon));
			Obj.Weapons[j]->x = x;
			Obj.Weapons[j]->y = y;
			Obj.Weapons[j]->fwcounter = 0.0f;
			Obj.Weapons[j]->WeaponType = &WeaponTypes[nWType];
		}
						
		// sorry for using x here, don't won't to create a new variable
		// FixMe: need a more elegant way
		x = create_object(&Obj);

		if (nObjType == 1)
		{
			num_player1 = x;
			printf("num_player1: %d\n",num_player1);
		}
	}
	
	fclose(file);
	return 0;
}

int Objects::fire_weapons(int nObj)
{
	unsigned int i,shot;
	SObject *Attacker;
	
	Attacker = &lObjects[nObj];
	
	for (i=0; i<Attacker->numWeapons; i++)
	{
		if (Attacker->Weapons[i]->fwcounter > Attacker->Weapons[i]->WeaponType->fwtime)
		{
			Attacker->Weapons[i]->fwcounter = 0.0f;
			SObject Obj;

			if(Attacker->Weapons[i]->WeaponType->nWType == WEAPON_NULL)
				continue;
	
			Obj.ObjType = &lObjectTypes[Attacker->Weapons[i]->WeaponType->nProjObj];
		
			Obj.x = Attacker->x+Attacker->Weapons[i]->x;
			Obj.y = Attacker->y+Attacker->Weapons[i]->y;

			Obj.nanim = 0;
			Obj.hp = Attacker->Weapons[i]->WeaponType->damage;
			Obj.numWeapons = 0;
		
			shot = create_object(&Obj);
			
			/* invert movement direction for non-player shots */
			if (Attacker->ObjType->nType == OBJ_TYPE_PLAYER)
			{
				g_pfaMain->g_pSound->play_sound(1);
			}
			else
			{
				lObjects[shot].speed_x = -lObjects[shot].speed_x;
			}
			
			// set parent AFTER creating object, because create_object sets it to NULL
			lObjects[shot].nparent = nObj;
		}
	}
	return 0;
}

int Objects::load_objecttypes(char* filename)
{
	int nObjectTypes;
	FILE* file;
	int nObjType,w,h,nImage,maxanim,maxhp,i,movement;
	float speed,animspeed,flifetime;
	char name[20];
	SObjectType	ObjType;
	
	file = fopen(filename,"r");
	if (file == NULL)
	{
		return 1;
	}
	fscanf(file,"%d\n",&nObjectTypes);
	printf("nObjectTypes: %d\n",nObjectTypes);
	
	for (i=1; i<nObjectTypes+1; i++)
	{
		fscanf(file,"%d\t%d\t%d\t%f\t%d\t%d\t%f\t%f\t%d\t%d\t%s\n",&nObjType,&w,&h,&speed,&nImage,&maxanim,&animspeed,&flifetime,&maxhp,&movement,name);
		printf("%d: %d\t%d\t%d\t%f\t%d\t%d\t%f\t%f\t%d\t%d\t%s\n",i,nObjType,w,h,speed,nImage,maxanim,animspeed,flifetime,maxhp,movement,name);

		ObjType.nType = nObjType;
		ObjType.w = w;
		ObjType.h = h;
		ObjType.speed = speed;
		ObjType.nImage = nImage;
		printf("nImage: %d\n",ObjType.nImage);
		ObjType.maxanim = maxanim;
		ObjType.animspeed = animspeed;
		ObjType.flifetime = flifetime;
		ObjType.maxhp = maxhp;
		ObjType.movement = movement;
		
		lObjectTypes[i].nType = 0;
				
		lObjectTypes[i] = ObjType;
		//printf("created objecttype\n");	
	}
	
	fclose(file);
	return 0;
}
	
int Objects::load_weapontypes(char* filename)
{
	int nWeaponTypes;
	FILE* file;
	int i,nWType,damage,nProjObj;
	float fwtime;
	SWeaponType	WeaponType;
	
	file = fopen(filename,"r");
	if (file == NULL)
	{
		return 1;
	}
	fscanf(file,"%d\n",&nWeaponTypes);
	printf("nWeaponTypes: %d\n",nWeaponTypes);
	
	for (i=0; i<nWeaponTypes; i++)
	{
		nWType = 0;
		fscanf(file,"%d\t%f\t%d\t%d\n",&nWType,&fwtime,&damage,&nProjObj);
		printf("%d: %d\t%f\t%d\t%d\n",i,nWType,fwtime,damage,nProjObj);
		printf("nproj: %d\n",nProjObj);
		WeaponType.nWType = nWType;
		WeaponType.fwtime = fwtime;
		WeaponType.damage = damage;
		WeaponType.nProjObj = nProjObj;
				
		WeaponTypes[i] = WeaponType;
		printf("added weapontype\n");	
	}
	
	fclose(file);
	return 0;
}


void Objects::action_kill(unsigned int nObj, bool do_anim)
{
	lObjects[nObj].nState = OBJ_STATE_DYING;
	lObjects[nObj].nanim = 0;

	if ( (lObjects[nObj].ObjType->nType == OBJ_TYPE_STONE 
		|| lObjects[nObj].ObjType->nType == OBJ_TYPE_ENEMY) && 
		lObjects[nObj].ObjType->maxhp > 50) {
		
		int chance = (rand()%10);
		
		float x,y;
		x = lObjects[nObj].x+(lObjects[nObj].ObjType->w/2);
		y = lObjects[nObj].y+(lObjects[nObj].ObjType->h/2);
		
		if(chance < 2) {
			drop_powerup(x, y, OBJ_TYPE_PWRUP_L1 + rand() % 3);
		} else if(chance < 3) {
			drop_powerup(x, y, OBJ_TYPE_PWRUP_MED);
		} else if(chance < 4) {
			drop_powerup(x, y, OBJ_TYPE_PWRUP_SPEED);
		} else if(chance < 5) {
			drop_powerup(x, y, OBJ_TYPE_PWRUP_REGENERATE);
		}
	}


	if(do_anim)
	{
		lObjects[nObj].nanimtype = 1;
	}
	else
	{
		destroy_object(nObj);
	}
}

void Objects::action_collide(unsigned int nObj1, unsigned int nObj2) {
	struct SObject *o1, *o2;
	
	o1 = &lObjects[nObj1];
	o2 = &lObjects[nObj2];
	
	// FixMe: really bad hack
	// no "friendly fire"
	if ( (lObjects[nObj1].nparent > 0) && (lObjects[nObj1].nparent != num_player1) ) {
		if ( (lObjects[nObj2].ObjType->nType == OBJ_TYPE_ENEMY) || (lObjects[nObj2].ObjType->nType == OBJ_TYPE_STONE) ){
			destroy_object(nObj1);
			return;
		}
	}
	
	if ( (lObjects[nObj2].nparent > 0) && (lObjects[nObj2].nparent != num_player1) ) {
		if ( (lObjects[nObj1].ObjType->nType == OBJ_TYPE_ENEMY) || (lObjects[nObj1].ObjType->nType == OBJ_TYPE_STONE) ){
			destroy_object(nObj2);
			return;
		}
	}

	o1->hp -= o2->hp;
	o2->hp -= (o1->hp+o2->hp);

	check_death(nObj1);
	check_death(nObj2);
}

void Objects::action_health(unsigned int nObj, int amount) {
	lObjects[nObj].hp += amount;
	if(lObjects[nObj].hp > lObjects[nObj].ObjType->maxhp) {
		lObjects[nObj].hp = lObjects[nObj].ObjType->maxhp;
	}
	check_death(nObj);
}

void Objects::check_death(unsigned int nObj) {
	if(lObjects[nObj].hp <= 0 && lObjects[nObj].nState != OBJ_STATE_DYING) {
		action_kill(nObj, (lObjects[nObj].ObjType->nType != OBJ_TYPE_SHOT));
	}
}

int Objects::clear_level()
{
	unsigned int	i, size = lObjects.size();
	
	for (i=0; i<size; i++)
	{
		if (NULL != lObjects[i].ObjType)
		{
			del_object(i);
		}
	}

	lObjects.clear();
	
	return 0;
}
	
	
void Objects::action_pwrup(unsigned int nObjPlayer, unsigned int nObjPwrup) {
	switch(lObjects[nObjPwrup].ObjType->nType) {
		case OBJ_TYPE_PWRUP_L1:
			/* fixme: hardcoded weapon type */
			install_weapon(nObjPlayer, 5);
			break;
		case OBJ_TYPE_PWRUP_L2:
			install_weapon(nObjPlayer, 6);
			break;
		case OBJ_TYPE_PWRUP_L3:
			install_weapon(nObjPlayer, 2);
			break;
		case OBJ_TYPE_PWRUP_SPEED:
			lObjects[nObjPlayer].pwrup_speed_seconds_left += 10;
			break;
		case OBJ_TYPE_PWRUP_REGENERATE:
			lObjects[nObjPlayer].pwrup_regenerate_seconds_left += 10;
			break;
	}
}

void Objects::install_weapon(unsigned int nObjPlayer, unsigned int weapon_num) {
	struct SObject *player = &lObjects[nObjPlayer];
	bool found = false;
	unsigned int i;

	/* find an empty slot */
	for(i=0; i < player->numWeapons; i++) {
		if(player->Weapons[i]->WeaponType->nWType == WEAPON_NULL) {
			found = true;
			break;
		}
	}
	
	if(!found) {
		printf("no free slot, sorry\n");
		return;
	}
	
	player->Weapons[i]->WeaponType = &WeaponTypes[weapon_num];
}

void Objects::drop_powerup(float x, float y, int type) {
	SObject Obj;
	unsigned int powerup, i;
	
	for(i = 1; i < MAX_OBJECT_TYPES; i++) {
		if(lObjectTypes[i].nType == type) {
			type = i;
			break;
		}
	}

	Obj.ObjType = &lObjectTypes[type];
	Obj.x = x-(Obj.ObjType->w/2);
	Obj.y = y-(Obj.ObjType->h/2);
	Obj.speed_x = 0;
	Obj.speed_y = 0;
	Obj.nanim = 0;
	Obj.hp = 1000;
	Obj.numWeapons = 0;
	
	powerup = create_object(&Obj);
}

int Objects::get_random(int a, int b)
{
	int result;
	result = rand()%(2*b)-10;
	
	if (result == 0) {
		if (rand()%2) {
			result = 1;
		} else {
			result = -1;
		}
	}
	
	return result;
}
	
	
	
	
