/**
far away - a free space shooter game
Copyright (C) 2005  Alexander Clausen, Dirk Rathlev, Stefan Toller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <iostream>
#include <list>
#include <vector>


using namespace std;

struct SObjectType
{
	// type of the object
	int	nType;
	// width and height in pixels
	int	w,h;
	// maximum speed
	float	speed;
	// image (index in images array)
	int	nImage;
	// number of available animation frames
	int	maxanim;
	// speed of animation
	float	animspeed;
	// lifetime of object
	float	flifetime;
	// max hitpoints
	int	maxhp;
	// how does it move?
	int movement;
};

struct SObject
{
	// pointer to type of the object
	SObjectType	*ObjType;
	// actual position of the object
	float	x,y;
	// actual speed
	float	speed_x,speed_y;
	// actual animation frame
	int	nanim;
	// actual animation type (moving, dieing, firing, ...)
	int	nanimtype;
	// internal counter for animations
	float	animtime;
	// internal counter, gets started when object gets created
	float	fcounter;
	// hitpoints
	int	hp;
	// actual state (attacking, dieing, ...)
	int	nState;
	// how many weapons?
	unsigned int	numWeapons;
	// weapons of the object
	SWeapon	*Weapons[MAX_ATTACHED_WEAPONS];
	// the "parent" of the object
	// used f. e. to avoid collisions between a ship and its own laser
	int	nparent;

	// powerup stuff - is this the best place?
	float pwrup_speed_seconds_left;
	float pwrup_regenerate_seconds_left;
};

class Objects{
	private:

	public:
		vector <SObject> lObjects;
		SObjectType lObjectTypes[MAX_OBJECT_TYPES];
		
		// number of 1st player in lObjects array
		unsigned int num_player1;
		
		// array of different weapon types
		SWeaponType WeaponTypes[MAX_WEAPON_TYPES];
		
		int objects_init();
		
		int add_objecttype(SObjectType *ObjType);
		int del_objecttype(int nObjType);
		
		int add_object(SObject *Obj);
		int del_object(int nObj);
		int exit_objects(void);
		int process_objects(void);
		
		// process ai for each object
		int ai_object(int i);
		int move_object(unsigned int i);
		int animate_object(int nObj);
		
		unsigned int check_collision_rect(int nObj1, int nObj2);
		int print_debug(void);
		int create_object(SObject *Obj);
		int destroy_object(int nObj);
		int load_level(char* filename);
		int fire_weapons(int nObj);
		int load_objecttypes(char* filename);
		int load_weapontypes(char* filename);
		void install_weapon(unsigned int nObjPlayer, unsigned int weapon_num); 
		
		void action_kill(unsigned int nObj, bool do_anim);
		void action_collide(unsigned int nObj1, unsigned int nObj2);
		void action_health(unsigned int nObj, int amount);
		void action_pwrup(unsigned int nObjPlayer, unsigned int nObjPwrup);
		void drop_powerup(float x, float y, int type);
		void check_death(unsigned int nObj);
		// deletes all objects to prepare for new level load
		int clear_level();
		// get a random number from a to b
		int get_random(int a, int b);
};
